import java.sql.*;
import java.util.LinkedList;

public class TimeTrackerDB {

	private java.sql.Connection conn;
	private String url;

	private static final int DUPKEY_ERRCODE = 1062;

	TimeTrackerDB() throws SQLException, ClassNotFoundException {
		try {

			/*
			//Load the hsqldb driver
			Class.forName("org.hsqldb.jdbcDriver");
			url = "jdbc:hsqldb:Employees"; // stored on disk mode
			//Open the connection
			conn = DriverManager.getConnection(url, "SA", "");
			System.out.println("Opened HSQLDB database.");
			*/

			//@Version ("0.2.1") - Now connect MySQL instead of MS Access
			//Load the MySQL driver
			Class.forName("com.mysql.jdbc.Driver");
			url = "jdbc:mysql://localhost/timetracker";
			//Open the connection
			conn = DriverManager.getConnection(url, "timetrack", "timetrack");
			System.out.println("Opened MYSQL database.");
			
			
			/*@Version ("0.2.1")
			//Load the MS Access drivers
			//Remember that in Windows 2000 and XP, ODBC connections are defined
			// in the Control Panel --> Admin Tools --> Data Sources
			Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
			url = "jdbc:odbc:TimeTracker";
			//Open the connection
			conn = DriverManager.getConnection(url, "", "");
			System.out.println("Opened ACCESS database.");
			*/

		} catch (Exception e) {
			System.out.println("*** Error: " + e.getMessage());
		}
	}

	public void close() throws SQLException {
		//Close the open database connection.
		conn.close();
		System.out.println("Closed the database.");
	}

	public Employee[] retrieveEmployees() throws SQLException {
		String query = "SELECT * FROM Employee;";
		ResultSet rs = retrieve(query);

		//Parse the ResultSet into a LinkedList of Employees
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next employee object, and load the fields.
			Employee c = new Employee();
			c.setFields(
				rs.getInt("employeeID"),
				rs.getString("firstName"),
				rs.getString("lastName"),
				rs.getString("phone"),
				rs.getString("email"),
				rs.getDate("dob").toString(),
				rs.getString("address"),
				rs.getString("miscInfo"));
			results.add(c);
		}

		Employee[] retval = new Employee[results.size()];
		retval = (Employee[])results.toArray(retval);

		return retval;
	}

	public Project[] retrieveProjects() throws SQLException {
		String query = "SELECT * FROM Project;";
		ResultSet rs = retrieve(query);

		//Parse the ResultSet into a LinkedList of Projects
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next project object, and load the fields.
			Project c = new Project();
			c.setFields(
				rs.getInt("projectID"),
				rs.getString("name"),
				rs.getInt("managerID"));
			results.add(c);
		}

		Project[] retval = new Project[results.size()];
		retval = (Project[])results.toArray(retval);

		return retval;
	}

	public Project[] retrieveProjectsForEmployee(int eID) throws SQLException {
		String query = "SELECT * FROM Project p, EmployeeProject_XREF e "
			+ "WHERE employeeID = "+ eID +" AND p.projectID = e.projectID;";
		ResultSet rs = retrieve(query);

		//Parse the ResultSet into a LinkedList of Projects
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next project object, and load the fields.
			Project c = new Project();
			c.setFields(
				rs.getInt("projectID"),
				rs.getString("name"),
				rs.getInt("managerID"));
			results.add(c);
		}

		Project[] retval = new Project[results.size()];
		retval = (Project[])results.toArray(retval);

		return retval;
	}
	
	public Employee[] retrieveEmployeesInProject(int pID) throws SQLException {
		String query = "SELECT * FROM Employee c, EmployeeProject_XREF e "
			+ "WHERE projectID = "+ pID +" AND c.employeeID = e.employeeID;";
		ResultSet rs = retrieve(query);

		//Parse the ResultSet into a LinkedList of Employees
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next employee object, and load the fields.
			Employee c = new Employee();
			c.setFields(
				rs.getInt("employeeID"),
				rs.getString("firstName"),
				rs.getString("lastName"),
				rs.getString("phone"),
				rs.getString("email"),
				rs.getDate("dob").toString(),
				rs.getString("address"),
				rs.getString("miscInfo"));
			results.add(c);
		}

		Employee[] retval = new Employee[results.size()];
		retval = (Employee[])results.toArray(retval);

		return retval;
	}
	
	public Employee[] retrieveEmployeesNotInProject(int pID) throws SQLException {
		String query = "SELECT * FROM Employee c WHERE NOT EXISTS "
			+ "(SELECT * FROM EmployeeProject_XREF e "
				+"WHERE projectID = " + pID + " AND c.employeeID = e.employeeID);";
		ResultSet rs = retrieve(query);

		//Parse the ResultSet into a LinkedList of Employees
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next employee object, and load the fields.
			Employee c = new Employee();
			c.setFields(
				rs.getInt("employeeID"),
				rs.getString("firstName"),
				rs.getString("lastName"),
				rs.getString("phone"),
				rs.getString("email"),
				rs.getDate("dob").toString(),
				rs.getString("address"),
				rs.getString("miscInfo"));
			results.add(c);
		}

		Employee[] retval = new Employee[results.size()];
		retval = (Employee[])results.toArray(retval);

		return retval;
	}

	public Transaction[] retrieveTransactions(int employeeID, int projectID)
		throws SQLException {
		//@Version ("0.2.3") - Changed FROM transaction to Transaction
		//@Version ("0.2.3") - Put one space before AND projectID
		String query =
			"SELECT * FROM Transaction where employeeID = "
				+ employeeID
				+ " AND projectID = "
				+ projectID
				+ " ORDER BY transactionID;";
		ResultSet rs = retrieve(query);

		//Build a LinkedList of the correct Transactions.
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next Transaction object, and load its fields.
			Transaction t = new Transaction();
			t.setFields(
				rs.getInt("transactionID"),
				rs.getInt("employeeID"),
				rs.getInt("projectID"),
				rs.getInt("hours"),
				rs.getDate("tDate").toString(),
				rs.getString("activity"));
			results.add(t);
		}

		//Convert the LinkedList to an array, and return it.
		Transaction[] retval = new Transaction[results.size()];
		retval = (Transaction[])results.toArray(retval);
		return retval;
	}

	public Transaction[] retrieveTransactions(int employeeID)
		throws SQLException {
		String query =
			"SELECT * FROM Transaction where employeeID = "
				+ employeeID
				+ " ORDER BY transactionID;";
		ResultSet rs = retrieve(query);

		//Build a LinkedList of the correct Transactions.
		LinkedList results = new LinkedList();
		while (rs.next()) {
			//Create the next Transaction object, and load its fields.
			Transaction t = new Transaction();
			t.setFields(
				rs.getInt("transactionID"),
				rs.getInt("employeeID"),
				rs.getInt("projectID"),
				rs.getInt("hours"),
				rs.getDate("tDate").toString(),
				rs.getString("activity"));
			results.add(t);
		}

		//Convert the LinkedList to an array, and return it.
		Transaction[] retval = new Transaction[results.size()];
		retval = (Transaction[])results.toArray(retval);
		return retval;
	}

	public boolean create(Employee c) {
		//Build an SQL insert statement for the new Employee.
		String update =
			"INSERT INTO Employee(employeeID, firstName, lastName, phone, email, dob, address, miscInfo) VALUES (";

		//Specify the fields.
		update	+=				c.employeeID			+ ", "
				+	"'" +		c.firstName		+ "'"	+ ", "
				+	"'" +		c.lastName		+ "'"	+ ", "	
				+	"'" +		c.phone			+ "'"	+ ", "
				+	"'" +		c.email			+ "'"	+ ", "
				+	"'" +		c.dob			+ "'"	+ ", "
				+	"'" +		c.address		+ "'"	+ ", "
				+	"'" +		c.miscInfo		+ "'"	+ ");";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) { //duplicate key
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}
	
	public boolean create(Project c) {
		//Build an SQL insert statement for the new Project.
		String update =
			"INSERT INTO Project(projectID, name, managerID) VALUES (";

		//Specify the fields.
		update	+=				c.projectID				+ ", "
				+	"'" +		c.name			+ "'"	+ ", "
				+	"'" +		c.managerID		+ "'"	+ ");";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) { //duplicate key
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}

	public boolean create(Transaction t) {
		//Build an SQL insert statement for the new Transaction.
		String update =
			"INSERT INTO Transaction(transactionID, employeeID, projectID, hours, tDate, activity) VALUES (";

		//Specify the fields.
		update	+=				t.transactionID				+ ", "
				+				t.employeeID				+ ", "
				+				t.projectID					+ ", "	
				+				t.hours						+ ", "
				+	"'" +		t.date				+ "'"	+ ", "
				+	"'" +		t.activity			+ "'"	+ ");";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) { //duplicate key
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}

	public boolean create(int employeeID, int projectID) {
		//Build an SQL insert statement for the new EmployeeProject_XREF.
		String update =
			"INSERT INTO EmployeeProject_XREF(employeeID, projectID) VALUES (";

		//Specify the fields.
		update	+=				employeeID					+ ", "
				+	"'" +		projectID			+ "'"	+ ");";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) { //duplicate key
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}

	public boolean update(int oldID, Employee c) {
		//Build an SQL update statement for the Employee.
		String update = "UPDATE Employee SET ";

		update	+=	"employeeID = "	+			c.employeeID			+ ", "
				+	"firstName = "	+	"'" +	c.firstName		+ "'"	+ ", "
				+	"lastName = "	+	"'" +	c.lastName		+ "'"	+ ", "
				+	"phone = "		+	"'" +	c.phone			+ "'"	+ ", "
				+	"email = "		+	"'" +	c.email			+ "'"	+ ", "
				+	"dob = "		+	"'" +	c.dob			+ "'"	+ ", "
				+	"address = "	+	"'" +	c.address		+ "'"	+ ", "
				+	"miscInfo = "	+	"'" +	c.miscInfo		+ "'"	+ " "

				+ 	"WHERE "
				+	"employeeID = "	+ 			oldID					+ ";";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) {
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}
	
	public boolean update(int oldID, Project c) {
		//Build an SQL update statement for the Project.
		String update =
			"UPDATE Project SET ";

		update	+=	"projectID = "	+			c.projectID				+ ", "
				+	"name = "		+	"'" +	c.name			+ "'"	+ ", "
				+	"managerID = "	+	"'" +	c.managerID		+ "'"	+ " "

				+ "WHERE "
				+	"projectID = "	+ 			oldID					+ ";";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) {
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}

	public boolean update(int oldID, Transaction t) {
		//Build an SQL update statement for the Transaction.
		String update =
			"UPDATE Transaction SET ";

		update	+=	"transactionID = "	+			t.transactionID				+ ", "
				+	"employeeID = "		+			t.employeeID				+ ", "
				+	"projectID = "		+			t.projectID					+ ", "
				+	"hours = "			+			t.hours						+ ", "
				+	"tDate = "			+	"'" +	t.date				+ "'"	+ ", "
				+	"activity = "		+	"'" +	t.activity			+ "'"	+ " "

				+ "WHERE "
				+	"transactionID = "	+ 			oldID						+ ";";

		try {
			execute(update);
		} catch (SQLException e) {
			if (e.getErrorCode() == DUPKEY_ERRCODE) {
				return false;
			} else
				e.printStackTrace();
		}
		return true;
	}

	public void delete(Employee c) throws SQLException {
		//Build and execute a SQL delete statement for the Employee.
		String query =
			"DELETE FROM Employee WHERE employeeID = " + c.employeeID + ";";
		execute(query);
	}
	
	public void delete(Project c) throws SQLException {
		//Build and execute a SQL delete statement for the Project.
		String query =
			"DELETE FROM Project WHERE projectID = " + c.projectID + ";";
		execute(query);
	}

	public void delete(Transaction t) throws SQLException {
		//Build and execute a SQL delete statement for the Transaction.
		String query =
			"DELETE FROM Transaction WHERE transactionID = " + t.transactionID + ";";
		execute(query);
	}
	
	public void delete(Employee c, Project p) throws SQLException {
		//Build and execute a SQL delete statement for the EmployeeProject_XREF.
		String query = 
			"DELETE FROM EmployeeProject_XREF WHERE employeeID = " + c.employeeID
			+ " AND projectID = " + p.projectID + ";";
		execute(query);
	}
	
	public void deleteTransactionsByEmployeeID(int employeeID) throws SQLException {
		//Build and execute a SQL delete statement for the Transaction.
		String query =
			"DELETE FROM Transaction WHERE employeeID = " + employeeID + ";";
		execute(query);
	}

	public void deleteTransactionsByProjectID(int projectID) throws SQLException {
		//Build and execute a SQL delete statement for the Transaction.
		String query =
			"DELETE FROM Transaction WHERE projectID = " + projectID + ";";
		execute(query);
	}
	
	public void deleteEmployeeProjectByEmployeeID(int employeeID) throws SQLException {
		//Build and execute a SQL delete statement for the EmployeeProject_XREF
		String query =
			"DELETE FROM EmployeeProject_XREF WHERE employeeID = " + employeeID + ";";
		execute(query);
	}

	public void deleteEmployeeProjectByProjectID(int projectID) throws SQLException {
		//Build and execute a SQL delete statement for the EmployeeProject_XREF
		String query =
			"DELETE FROM EmployeeProject_XREF WHERE projectID = " + projectID + ";";
		execute(query);
	}
	
	public boolean isEmployeeInProject(int employeeID, int projectID) {
		ResultSet rs = null;
		boolean result = false;
		
		//Build a query to see if the Employee is in the Project. 
		String query = "SELECT * FROM EmployeeProject_XREF WHERE "
			+ 	"employeeID = "		+	employeeID		+	" AND "
			+	"projectID = "		+	projectID		+	";";
		
		try {
			rs = retrieve(query);

			if (rs != null && rs.next())
				result = true;
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return result;
	}

	private void execute(String cmd) throws SQLException {
		//Execute a SQL statement.

		Statement s = conn.createStatement();
		s.executeUpdate(cmd);
	}

	private ResultSet retrieve(String query) throws SQLException {
		//Execute a SQL statement, usually a SELECT, which returns a ResultSet.
		Statement s = conn.createStatement();
		ResultSet res = s.executeQuery(query);
		return res;
	}
}
